import com.urbancode.anthill3.AHPTool;
import com.urbancode.anthill3.XTrustProvider;
import com.urbancode.anthill3.plugin.jira.addcomments.FailMode;

XTrustProvider.install();

final def ahptool = new AHPTool()
final File resourceHome = new File(System.getenv()['PLUGIN_HOME'])
final def buildLifeId = System.getenv()['AH_BUILD_LIFE_ID']

final def props = ahptool.getStepProperties()
final def serverUrl       = props['integration/serverUrl']
final def serverVersion   = props['integration/serverVersion'];
final def username        = props['integration/username']
final def password        = props['integration/password']

final def issueKeyPattern = props['issueKeyPattern'];
final def commentBody     = props['commentBody']
final def failMode        = FailMode.valueOf(props['failMode']);
//------------------------------------------------------------------------------
// Script content
//------------------------------------------------------------------------------

println "Server:\t$serverUrl";
println "Issue Key Pattern:\t$issueKeyPattern"
println "Comment:\t$commentBody"
println "";

// Load the appropriate Jira Soap Classes into classloader
final def jiraSoapJar = new File(resourceHome, "jira-${serverVersion}.wsdl.jar")
assert jiraSoapJar.isFile();
this.class.classLoader.rootLoader.addURL(jiraSoapJar.toURL())
def JiraSoapServiceServiceLocator = Class.forName('com.atlassian.jira.rpc.soap.jirasoapservice_v2.JiraSoapServiceServiceLocator');
def RemoteComment = Class.forName('com.atlassian.jira.rpc.soap.beans.RemoteComment')

// get the changes of the build life from ahptool
def issueIds = new HashSet(ahptool.getChangeSets(issueKeyPattern).keySet())
println "Found ${issueIds.size()} issue keys";
if (issueIds.isEmpty()) {
    println "No issue keys found in changelog, either there were no changes or GetChangeLogStep was not run.";
    return;
}

def issueNotFoundExceptions = []; // for non FAIL_FAST modes

// connect to Jira
String fullAddress = serverUrl + "/rpc/soap/jirasoapservice-v2";
if (!fullAddress.startsWith("http")) {
    fullAddress = "http://" + fullAddress;
}
def locator = JiraSoapServiceServiceLocator.newInstance();
locator.jirasoapserviceV2EndpointAddress = fullAddress;
def jiraSoapService = locator.getJirasoapserviceV2();
def sessionToken = jiraSoapService.login(username, password);
try {
    for (def issueId : issueIds.sort()) {
        def comment = RemoteComment.newInstance();
        comment.setBody(commentBody);
        try {
            jiraSoapService.addComment(sessionToken, issueId, comment);
            println("Successfully added new comment to issue " + issueId);
        }
        catch (Exception e) {
            boolean isNotFoundException = e.message?.startsWith("Could not locate issue with id");
            if (isNotFoundException) {
                println("Issue "+issueId+" not found");
                issueNotFoundExceptions << e;
                if (failMode == FailMode.FAIL_FAST) {
                    throw e;
                }
            }
            else {
                throw e;
            }
        }
    }
}
finally {
    jiraSoapService.logout(sessionToken)
}

// check post conditions
if (failMode == FailMode.FAIL_ON_NO_UPDATES && !issueIds) {
    throw new IllegalStateException("No issues found to update.");
}
if (!issueNotFoundExceptions) {
    println("Failed to add comments to "+issueNotFoundExceptions.size()+" issues");
    if (failMode != FailMode.WARN_ONLY) {
        // throw the first exception we found as our failure
        throw issueNotFoundExceptions.iterator().next();
    }
}
