import com.urbancode.anthill3.AHPTool;
import com.urbancode.anthill3.XTrustProvider;
import com.urbancode.anthill3.plugin.jira.addcomments.FailMode;

XTrustProvider.install();

final def ahptool = new AHPTool()
final File resourceHome = new File(System.getenv()['PLUGIN_HOME'])
final def buildLifeId = System.getenv()['AH_BUILD_LIFE_ID']

final def props = ahptool.getStepProperties()
final def serverUrl       = props['integration/serverUrl']
final def serverVersion   = props['integration/serverVersion'];
final def username        = props['integration/username']
final def password        = props['integration/password']
final def projectKey = props['projectKey']
final def assignee = props['assignee']
final def summary = props['summary']
final def environment = props['environment']
final def issueDescription = props['issueDescription']

// CONSTANTS
final def issueTypeId = "1";                    // BUG
final def priorityId  = "4";                    // minor
final def dueDate     = Calendar.getInstance(); // right now

//------------------------------------------------------------------------------
// Script content
//------------------------------------------------------------------------------

println "Server:\t$serverUrl";
println "Project:\t$projectKey";
println "";

// Load the appropriate Jira Soap Classes into classloader
final def jiraSoapJar = new File(resourceHome, "jira-${serverVersion}.wsdl.jar")
assert jiraSoapJar.isFile();
this.class.classLoader.rootLoader.addURL(jiraSoapJar.toURL())
def JiraSoapServiceServiceLocator = Class.forName('com.atlassian.jira.rpc.soap.jirasoapservice_v2.JiraSoapServiceServiceLocator');
def RemoteIssue = Class.forName('com.atlassian.jira.rpc.soap.beans.RemoteIssue')

// connect to Jira
String fullAddress = serverUrl + "/rpc/soap/jirasoapservice-v2";
if (!fullAddress.startsWith("http")) {
    fullAddress = "http://" + fullAddress;
}
def locator = JiraSoapServiceServiceLocator.newInstance();
locator.jirasoapserviceV2EndpointAddress = fullAddress;
def jiraSoapService = locator.getJirasoapserviceV2();
def sessionToken = jiraSoapService.login(username, password);
try {
    def issue = RemoteIssue.newInstance()
    issue.project  = projectKey
    issue.type     = issueTypeId
    issue.priority = priorityId
    issue.duedate  = dueDate
    issue.assignee = assignee

    if (summary) {
        issue.summary = summary
    }
    if (environment) {
        issue.environment = environment
    }
    if (issueDescription) {
        issue.description = issueDescription
    }

    def resultIssue = jiraSoapService.createIssue(sessionToken, issue)
    println "Successfully created issue $resultIssue.key";
    def status = jiraSoapService.getStatuses(sessionToken).find{it.id == resultIssue.status};
    println "\tStatus Name = $status.name";
    def resolution = jiraSoapService.getResolutions(sessionToken).find{it.id == resultIssue.resolution};
    println "\tResolution Name = ${resolution?.name  ?: 'Unresolved'}"
}
finally {
    jiraSoapService.logout(sessionToken)
}
