
import groovy.xml.StreamingMarkupBuilder;

import com.urbancode.anthill3.AHPTool;
import com.urbancode.anthill3.XTrustProvider;
import com.urbancode.anthill3.plugin.jira.addcomments.FailMode;

XTrustProvider.install();

final def ahptool = new AHPTool()
final File resourceHome = new File(System.getenv()['PLUGIN_HOME'])
final def buildLifeId = System.getenv()['AH_BUILD_LIFE_ID']

final def props = ahptool.getStepProperties()
final def serverUrl       = props['integration/serverUrl']
final def serverVersion   = props['integration/serverVersion'];
final def username        = props['integration/username']
final def password        = props['integration/password']

final def issueKeyPattern = props['issueKeyPattern'];

//------------------------------------------------------------------------------
// Script content
//------------------------------------------------------------------------------

println "Server:\t$serverUrl";
println "Issue Key Pattern:\t$issueKeyPattern"
println "";

//Load the appropriate Jira Soap Classes into classloader
final def jiraSoapJar = new File(resourceHome, "jira-${serverVersion}.wsdl.jar")
assert jiraSoapJar.isFile();
this.class.classLoader.rootLoader.addURL(jiraSoapJar.toURL())
def JiraSoapServiceServiceLocator = Class.forName('com.atlassian.jira.rpc.soap.jirasoapservice_v2.JiraSoapServiceServiceLocator');

//get the changes of the build life from ahptool
// TODO get changes with dependencies?
// TODO handle multiple changesets with same issue reference?
def idToChangeset = ahptool.getMatchingChangeSets(issueKeyPattern)
println "Found ${idToChangeset.size()} issue keys";
if (idToChangeset.isEmpty()) {
   println "No issue keys found in changelog, either there were no changes or GetChangeLogStep was not run.";
   return;
}


// connect to Jira
String fullAddress = serverUrl + "/rpc/soap/jirasoapservice-v2";
if (!fullAddress.startsWith("http")) {
    fullAddress = "http://" + fullAddress;
}
def locator = JiraSoapServiceServiceLocator.newInstance();
locator.jirasoapserviceV2EndpointAddress = fullAddress;
def jiraSoapService = locator.getJirasoapserviceV2();
def sessionToken = jiraSoapService.login(username, password);
try {
    def statusLookup = jiraSoapService.getStatuses(sessionToken).toList().groupBy{it.id}
    def issueTypeLookup = jiraSoapService.getIssueTypes(sessionToken).toList().groupBy{it.id}

    def builder = new StreamingMarkupBuilder()
    builder.encoding = 'UTF-8'
    def issuesXml = builder.bind {
        issues() {
            for (def issueKeyMapping in idToChangeset) {
                def issueKey = issueKeyMapping.key
                def changeId = issueKeyMapping.value.'anthill-id'.text();

                def jiraIssue = jiraSoapService.getIssue(sessionToken, issueKey)
                if (!jiraIssue) {
                    println("\tSkipping Issue "+issueKey+": Specified Issue not found.");
                }
                else {
                    issue(id:issueKey, "issue-tracker":"JIRA", "change-id":changeId) {
                        name(jiraIssue.summary)
                        description(jiraIssue.description)
                        type(issueTypeLookup[jiraIssue.type]?.name)
                        status(statusLookup[jiraIssue.status]?.name)
                    }
                }
            }
        }
    }

    ahptool.addIssues(issuesXml)
    println("Results uploaded successfully")
}
finally {
    jiraSoapService.logout(sessionToken)
}
